/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mdatool.addition;

import java.util.ArrayList;
import java.util.List;

import org.tinygroup.mda.config.entity.DisplayField;
import org.tinygroup.mda.config.entity.EntityField;
import org.tinygroup.mda.config.entity.EntityGroup;
import org.tinygroup.mda.config.entity.EntityModel;
import org.tinygroup.mda.config.entity.View;
import org.tinygroup.mda.config.entity.ViewGroup;

public final class DefaultView {
	/**
	 * 默认视图名
	 */
	private static String dVIEWNAME = "默认视图";
	/**
	 * 默认视图id
	 */
	private static String dVIEWID = "defaultmodleview";
	/**
	 * 表格默认单页数据条数
	 */
	private static int dPAGESIZE = 10;

	private DefaultView(){
		
	}
	/**
	 * 为模型生成基本视图，包含所有的模型字段及生成的默认的增删改查操作
	 * @param model
	 */
	public static void createDefaultView(EntityModel model) {
		
		View view = new View();
		view.setTitle(dVIEWNAME);
		view.setName(dVIEWID);
		view.setUuid(dVIEWID);
		view.setEnableDelete(true);
		view.setEnableModify(true);
		view.getPageInfo().setPageSize(dPAGESIZE);
		createDefaultViewOp(view);
		view.getViewGroups().addAll(createOpFiledGroups(model.getGroups()));
		//将model原有的view全部清除
		model.getViews().clear();
		//将默认view添加至模型
		model.getViews().add(view);
	}

	/**
	 * 为视图添加默认的增删改查操作引用
	 * @param view
	 */
	private static void createDefaultViewOp(View view) {
		view.getOperations().add(DefaultOperation.DEFAULT_ADD_OP_UUID);
		view.getOperations().add(DefaultOperation.DEFAULT_DELETE_OP_UUID);
		view.getOperations().add(DefaultOperation.DEFAULT_QUERY_OP_UUID);
		view.getOperations().add(DefaultOperation.DEFAULT_UPDATE_OP_UUID);
		view.getOperations().add(DefaultOperation.DEFAULT_VIEW_OP_UUID);
	}

	/**
	 * 根据模型字段分组列表生成视图字段分组列表
	 * @param groups
	 * @return
	 */
	private static List<ViewGroup> createOpFiledGroups(List<EntityGroup> groups) {
		List<ViewGroup> vGroups = new ArrayList<ViewGroup>();
		for (EntityGroup group : groups) {
			vGroups.add(createOpFiledGroups(group));
		}
		return vGroups;
	}

	/**
	 * 根据模型字段分组生成视图字段分组
	 * @param group
	 * @return
	 */
	private static ViewGroup createOpFiledGroups(EntityGroup group) {
		ViewGroup vGroup = new ViewGroup();
		vGroup.getFields().addAll(createDisplayFields(group.getFields()));
		vGroup.setName(group.getName());
		vGroup.setTitle(group.getTitle());
		vGroup.setUuid(getViewGroupUUID(group));
		vGroup.setEnableDelete(group.getEnableDelete());
		vGroup.setEnableModify(group.getEnableModify());
		return vGroup;
	}

	/**
	 * 根据模型字段列表获取视图字段列表
	 * @param entityFields
	 * @return
	 */
	private static List<DisplayField> createDisplayFields(
			List<EntityField> entityFields) {
		List<DisplayField> operationFields = new ArrayList<DisplayField>();
		for (EntityField entityField : entityFields) {
			DisplayField f = new DisplayField();
			f.setFielduuid(entityField.getUuid());
			operationFields.add(f);
		}
		return operationFields;
	}

	/**
	 * 获取视图字段列表uuid
	 * @param group
	 * @return
	 */
	private static String getViewGroupUUID(EntityGroup group) {
		return String.format("view%s", group.getUuid());
	}
}
