/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mdatool.util;

import java.io.File;
import java.util.List;

import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.mdatool.addition.FastModelDealer;

public final class ModelAdditionUtil {
	private static Logger logger = LoggerFactory.getLogger(ModelAdditionUtil.class);
	private ModelAdditionUtil() {
	}

	public static void main(String[] args) {
		try {
			if(args.length>=1){
				fastCreateModelInfo(null, args[0],null);
			}else{
				fastCreateModelInfo(null, null,null);
			}
		} catch (Throwable e) {
			logger.errorMessage(e.getMessage(), e);
		}
		
		
		
	}

	/**
	 * @param findPath 扫描的文件或者文件路径，若为空，则直接扫描该工程目录
	 * @param modelName 模型名称
	 * 					若非空，则为该模型生成默认视图和操作
	 * 					若为空，则为所有模型均生成默认视图和操作
	 * @param classPaths 添加额外的classpaths路径
	 */
	public static void fastCreateModelInfo(String findPath,String modelName,List<String> classPaths)  {
		ModelAdditionInitUtil.init(classPaths);
		String path = findPath;
		if(path==null||"".equals(path)){
			//获取当前工程目录
			path = new File("").getAbsolutePath().toString();
		}
		logger.logMessage(LogLevel.DEBUG,"FastModelUtil开始扫描，准备为模型生成默认视图及操作,modelName:{0};path:{1}",
				modelName,path);
		FastModelDealer.getInstance().find(path, modelName);
		logger.logMessage(LogLevel.DEBUG,"FastModelUtil处理完成,modelName:{0};path:{1}",
				modelName,path);
		
	}

}
